/* ----------------------------------------------------------------------- */
/*
 * Program:
 * Brute force method of determining the prime numbers in a given range
 *
 * Purpose:
 * Use Intel Thread Checker to identify threading errors
 *
 * Original source code:
 * Intel(R) Software College 
 *
 * Modifications:
 * Alexander Pozdneev
 */
/* ----------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
/* ----------------------------------------------------------------------- */
/* An array to store primes */
/* `glb' stands for `global scope variable' */
#define MAX_PRIMES 1000000
static long glb_primes[MAX_PRIMES];

/* Total number of primes found */
static int glb_primes_found = 0;

/* User-specified range (`cl' stands for `command line') */
static int cl_start, cl_end;
/* ----------------------------------------------------------------------- */
static void Get_CL_Arguments(int argc, char **argv, int *start, int *end)
{
    if (argc == 3)
    {
        cl_start = atoi(argv[1]);
        cl_end   = atoi(argv[2]);
    }
    else
    {
        printf("Usage: %s <start range> <end range>\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    if (cl_start > cl_end)
    {
        printf("Start value must be less than or equal to the end one\n");
        exit(EXIT_FAILURE);
    }
    if (cl_start < 1 && cl_end < 2)
    {
        printf("Range must be positive integers\n");
        exit(EXIT_FAILURE);
    }

    *start = cl_start;
    *end = cl_end;

    if (cl_start < 2) *start = 2;
    if (*start <= 2) glb_primes[glb_primes_found++] = 2;
}
/* ----------------------------------------------------------------------- */
static void Show_Progress(int range)
{
    int percent_done;
    static int progress = 0;
    static int last_percent_done = 0;

#pragma omp atomic
    ++progress;

    percent_done = (int)(progress/(double)range * 200.0 + 0.5);

    if (percent_done % 10 == 0 && last_percent_done < percent_done / 10) 
    {
        printf("\b\b\b\b%3d%%", percent_done);
        ++last_percent_done;
    }
}
/* ----------------------------------------------------------------------- */
static int Is_Prime(int val)
{
    int factor = 3;
    int limit = (int)(sqrt(val) + 0.5);

    while ( (factor <= limit) && (val % factor != 0) )
    {
        ++factor;
    }

    return (factor > limit);
}
/* ----------------------------------------------------------------------- */
static void Find_Primes(int start, int end)
{
    int range = end - start + 1; 
    int i;

#pragma omp parallel for
    for (i = start; i <= end; i += 2)
    {
        if (Is_Prime(i))
        {
            glb_primes[glb_primes_found++] = i;
        }
        Show_Progress(range);
    }
}
/* ----------------------------------------------------------------------- */
int main(int argc, char **argv)
{
    int start, end;
    clock_t before, after;

    Get_CL_Arguments(argc, argv, &start, &end);

    /* Ensure we start with an odd number */
    if ((start % 2) == 0)
    {
        ++start;
    }

    before = clock();
    Find_Primes(start, end);
    after = clock();

    printf("\n\n%8d primes found between %8d and %8d in %7.2f secs\n",
            glb_primes_found,
            cl_start,
            cl_end,
            (after - before)/(double)CLOCKS_PER_SEC
    );

    return EXIT_SUCCESS;
}
/* ----------------------------------------------------------------------- */
